from docuseal._http import DocusealHttp


class DocusealApi:
    def __init__(self, key=None, url="https://api.docuseal.com", read_timeout=60, open_timeout=60):
        self.config = {}

        self.config["key"] = key
        self.config["url"] = url
        self.config["read_timeout"] = read_timeout
        self.config["open_timeout"] = open_timeout

        self.http = DocusealHttp(self.config)

    @property
    def key(self):
        return self.config["key"]

    @key.setter
    def key(self, value):
        self.config["key"] = value

    @property
    def url(self):
        return self.config["url"]

    @url.setter
    def url(self, value):
        self.config["url"] = value

    @property
    def read_timeout(self):
        return self.config["read_timeout"]

    @read_timeout.setter
    def read_timeout(self, value):
        self.config["read_timeout"] = value

    @property
    def open_timeout(self):
        return self.config["open_timeout"]

    @open_timeout.setter
    def open_timeout(self, value):
        self.config["open_timeout"] = value

    def list_templates(self, params=None):
        if params is None:
            params = {}
        return self.http.get("/templates", params)

    def get_template(self, template_id, params=None):
        if params is None:
            params = {}
        return self.http.get(f"/templates/{template_id}", params)

    def create_template_from_docx(self, data):
        return self.http.post("/templates/docx", data)

    def create_template_from_html(self, data):
        return self.http.post("/templates/html", data)

    def create_template_from_pdf(self, data):
        return self.http.post("/templates/pdf", data)

    def merge_templates(self, data):
        return self.http.post("/templates/merge", data)

    def clone_template(self, template_id, data):
        return self.http.post(f"/templates/{template_id}/clone", data)

    def update_template(self, template_id, data):
        return self.http.put(f"/templates/{template_id}", data)

    def update_template_documents(self, template_id, data):
        return self.http.put(f"/templates/{template_id}/documents", data)

    def archive_template(self, template_id):
        return self.http.delete(f"/templates/{template_id}")

    def permanently_delete_template(self, template_id):
        return self.http.delete(f"/templates/{template_id}", {"permanently": True})

    def list_submissions(self, params=None):
        if params is None:
            params = {}
        return self.http.get("/submissions", params)

    def get_submission(self, submission_id, params=None):
        if params is None:
            params = {}
        return self.http.get(f"/submissions/{submission_id}", params)

    def create_submission(self, data):
        return self.http.post("/submissions/init", data)

    def create_submission_from_emails(self, data):
        return self.http.post("/submissions/emails", data)

    def archive_submission(self, submission_id):
        return self.http.delete(f"/submissions/{submission_id}")

    def permanently_delete_submission(self, submission_id):
        return self.http.delete(f"/submissions/{submission_id}", {"permanently": True})

    def list_submitters(self, params=None):
        if params is None:
            params = {}
        return self.http.get("/submitters", params)

    def get_submitter(self, submitter_id, params=None):
        if params is None:
            params = {}
        return self.http.get(f"/submitters/{submitter_id}", params)

    def update_submitter(self, submitter_id, data):
        return self.http.put(f"/submitters/{submitter_id}", data)




import json
import urllib.parse
import http.client
from docuseal._version import VERSION


class ApiError(Exception):
    pass


class DocusealHttp:
    BODY_METHODS = {"POST", "PUT"}

    def __init__(self, config):
        self.config = config

    def get(self, path, params=None):
        return self.send_request("GET", path, params)

    def post(self, path, body=None):
        return self.send_request("POST", path, None, body)

    def put(self, path, body=None):
        return self.send_request("PUT", path, None, body)

    def delete(self, path, params=None):
        return self.send_request("DELETE", path, params)

    def send_request(self, method, path, params=None, body=None):
        if params is None:
            params = {}

        url = urllib.parse.urljoin(self.config["url"], path)
        full_path = f"{url}{self.to_query(params)}"

        parsed_url = urllib.parse.urlparse(full_path)
        connection_class = (
            http.client.HTTPSConnection
            if parsed_url.scheme == "https"
            else http.client.HTTPConnection
        )
        conn = connection_class(
            parsed_url.hostname or "", parsed_url.port, timeout=self.config.get("open_timeout", 60)
        )

        headers = self.headers()
        data = json.dumps(body) if body is not None else None

        conn.request(method, f"{parsed_url.path}?{parsed_url.query}", body=data, headers=headers)
        response = conn.getresponse()

        conn.sock.settimeout(self.config.get("read_timeout", 60))

        return self.handle_response(response)

    def headers(self):
        return {
            "X-Auth-Token": self.config["key"] or "",
            "Content-Type": "application/json",
            "User-Agent": f"DocuSeal Python v{VERSION}",
        }

    def to_query(self, params):
        if not params:
            return ""
        return f"?{urllib.parse.urlencode(params)}"

    def handle_response(self, response):
        response_body = response.read().decode("utf-8")
        if 200 <= response.status < 300:
            return json.loads(response_body)
        else:
            raise ApiError(f"API Error {response.status}: {response_body}")



from setuptools import setup

setup(
    name='docuseal',
    version='1.0.1',
    description='DocuSeal Python API client',
    url='https://github.com/docusealco/docuseal-python',
    author='DocuSeal',
    author_email='admin@docuseal.com',
    keywords="docuseal document signature",
    license='MIT',
    packages=['docuseal'],
    install_requires=[],
    python_requires=">=3.6",
    project_urls={
        "Bug Tracker": "https://github.com/docuseaoco/docuseal-python/issues",
        "Changes": "https://github.com/docuseaoco/docuseal-python/blob/master/CHANGELOG.md",
        "Documentation": "https://www.docuseal.com/docs/api?lang=python",
        "Source Code": "https://github.com/docuseaoco/docuseal-python",
    },
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3 :: Only",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3.12",
        "Programming Language :: Python :: Implementation :: CPython",
        "Programming Language :: Python :: Implementation :: PyPy",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
    setup_requires=["wheel"],
)
